/*************************************************************
 * Author:	wangjiangyuan (wangjiangyuan@agora.io)
 * Date	 :	Oct 21th, 2020
 * Module:	Agora SD-RTN SDK RTC C API demo application.
 *
 *
 * This is a part of the Agora RTC Service SDK.
 * Copyright (C) 2020 Agora IO
 * All rights reserved.
 *
 *************************************************************/

#include <string.h>

#include "file_parser_priv.h"

extern media_parser_t video_parser_h264;
extern media_parser_t video_parser_yuv420;
extern media_parser_t audio_parser_aac;
extern media_parser_t audio_parser_pcm;

static media_parser_t *gs_media_parser_tab[] = {
	&video_parser_h264,
	&video_parser_yuv420,
	&audio_parser_aac,
	&audio_parser_pcm,
};
static int32_t gs_media_parser_cnt = sizeof(gs_media_parser_tab) / sizeof(gs_media_parser_tab[0]);

void *create_file_parser(int32_t codec, const char *path, parser_cfg_t *p_parser_cfg)
{
	int32_t i;
	for (i = 0; i < gs_media_parser_cnt; i++) {
		if (codec == gs_media_parser_tab[i]->codec) {
			break;
		}
	}

	if (i == gs_media_parser_cnt) {
		// codec not found, return NULL
		LOGE("Can't find the file parser matching codec type %d", codec);
		return NULL;
	}

	// codec found, now create the File parser
	media_parser_t *parser = (media_parser_t *)malloc(sizeof(media_parser_t));
	if (parser == NULL) {
		return NULL;
	}
	memcpy(parser, gs_media_parser_tab[i], sizeof(media_parser_t));
	if (p_parser_cfg) {
		memcpy(&(parser->parser_cfg), p_parser_cfg, sizeof(parser_cfg_t));
	}

	if (parser->open(parser, path) < 0) {
		free(parser);
		LOGE("File parser can't open file %s", path);
		return NULL;
	}

	return (void *)parser;
}

void destroy_file_parser(void *p_parser)
{
	if (p_parser) {
		media_parser_t *parser = (media_parser_t *)p_parser;
		parser->close(parser);
		free(parser);
	}
}

int32_t file_parser_obtain_frame(void *p_parser, frame_t *p_frame)
{
	int ret;
	if (!p_parser) {
		return -1;
	}

	media_parser_t *parser = (media_parser_t *)p_parser;

	ret = parser->obtain_frame(parser, p_frame);
	if (ret == -2) {
		parser->reset(parser);
		LOGI("File parser has reached the end of file. Now rewind ...");
		ret = parser->obtain_frame(parser, p_frame);
	}

	return ret;
}
